@@ -263,28 +263,29 @@ struct	ConfItem	{
 
 #define	CONF_ILLEGAL		0x80000000
 #define	CONF_MATCH		0x40000000
-#define	CONF_QUARANTINED_SERVER	0x00001
-#define	CONF_CLIENT		0x00002
-#define CONF_RCLIENT            0x00004
-#define	CONF_CONNECT_SERVER	0x00008
-#define	CONF_NOCONNECT_SERVER	0x00010
-#define	CONF_ZCONNECT_SERVER	0x00020
-#define	CONF_LOCOP		0x00040
-#define	CONF_OPERATOR		0x00080
-#define	CONF_ME			0x00100
-#define	CONF_KILL		0x00200
-#define	CONF_ADMIN		0x00400
+#define	CONF_QUARANTINED_SERVER	0x000001
+#define	CONF_CLIENT		0x000002
+#define CONF_RCLIENT            0x000004
+#define	CONF_CONNECT_SERVER	0x000008
+#define	CONF_NOCONNECT_SERVER	0x000010
+#define	CONF_ZCONNECT_SERVER	0x000020
+#define	CONF_LOCOP		0x000040
+#define	CONF_OPERATOR		0x000080
+#define	CONF_ME			0x000100
+#define	CONF_KILL		0x000200
+#define	CONF_ADMIN		0x000400
 #ifdef 	R_LINES
-#define	CONF_RESTRICT		0x00800
+#define	CONF_RESTRICT		0x000800
 #endif
-#define	CONF_CLASS		0x01000
-#define	CONF_SERVICE		0x02000
-#define	CONF_LEAF		0x04000
-#define	CONF_LISTEN_PORT	0x08000
-#define	CONF_HUB		0x10000
-#define	CONF_VER		0x20000
-#define	CONF_BOUNCE		0x40000
-#define	CONF_OTHERKILL		0x80000
+#define	CONF_CLASS		0x001000
+#define	CONF_SERVICE		0x002000
+#define	CONF_LEAF		0x004000
+#define	CONF_LISTEN_PORT	0x008000
+#define	CONF_HUB		0x010000
+#define	CONF_VER		0x020000
+#define	CONF_BOUNCE		0x040000
+#define	CONF_OTHERKILL		0x080000
+#define	CONF_DENY		0x100000
 
 #define	CONF_OPS		(CONF_OPERATOR | CONF_LOCOP)
 #define	CONF_SERVER_MASK	(CONF_CONNECT_SERVER | CONF_NOCONNECT_SERVER |\
@@ -378,6 +379,9 @@ struct	Server	{
 	aClient	*bcptr;
 	char	by[NICKLEN+1];
 	char	tok[5];
+	time_t	lastload;	/* penalty like counters, see s_serv.c
+				** should be in the local part, but..
+				*/
 };
 
 struct	Service	{
@@ -491,6 +495,12 @@ struct	stats {
 	u_long	is_lkt;   /* sum of elapsed time on when overwriting locked[]*/
 	u_long	is_lkMt;  /* max elapsed time on when overwriting locked[] */
 	u_long	is_lkmt;  /* min elapsed time on when overwriting locked[] */
+	u_int	is_ckl;   /* calls to check_link() */
+	u_int	is_cklQ;  /* rejected: SendQ too high */
+	u_int	is_ckly;  /* rejected: link too young */
+	u_int	is_cklno; /* rejected: "flood" */
+	u_int	is_cklok; /* accepted */
+	u_int	is_cklq;  /* accepted early */
 };
 
 /* mode structure for channels */
@@ -704,8 +714,9 @@ typedef	struct	{
 /* used for sendto_serv */
 
 /* semi-obsolete, bitmasks should now be used!!! */
-#define	SV_OLD	0x0000
-#define	SV_29	0x0001	/* useless, but preserved for coherence */
+#define	SV_OLD		0x0000
+#define	SV_29		0x0001	/* useless, but preserved for coherence */
+#define	SV_NJOIN	0x0002	/* server understands the NJOIN command */
 
 /* used for sendto_flag */
 
