@@ -18,7 +18,7 @@
  */
 
 #ifndef lint
-static  char rcsid[] = "@(#)$Id: support.c,v 1.9 1997/09/23 15:36:20 kalt Exp $";
+static  char rcsid[] = "@(#)$Id: support.c,v 1.10 1998/02/17 20:27:53 kalt Exp $";
 #endif
 
 #include "os.h"
@@ -35,6 +35,10 @@ static  char rcsid[] = "@(#)$Id: support.c,v 1.9 1997/09/23 15:36:20 kalt Exp $"
 #endif
 #undef SUPPORT_C
 
+#ifdef _WIN32
+int w32_h_errno = 0;
+#endif
+
 char	*mystrdup(s)
 char	*s;
 {
@@ -54,7 +58,7 @@ char	*s;
 **			of separators
 **			argv 9/90
 **
-**	$Id: support.c,v 1.9 1997/09/23 15:36:20 kalt Exp $
+**	$Id: support.c,v 1.10 1998/02/17 20:27:53 kalt Exp $
 */
 
 char *strtoken(save, str, fs)
@@ -108,7 +112,7 @@ char *str, *fs;
 **	strerror - return an appropriate system error string to a given errno
 **
 **		   argv 11/90
-**	$Id: support.c,v 1.9 1997/09/23 15:36:20 kalt Exp $
+**	$Id: support.c,v 1.10 1998/02/17 20:27:53 kalt Exp $
 */
 
 char *strerror(err_no)
@@ -138,7 +142,7 @@ int err_no;
 **			internet number (some ULTRIX don't have this)
 **			argv 11/90).
 **	inet_ntoa --	its broken on some Ultrix/Dynix too. -avalon
-**	$Id: support.c,v 1.9 1997/09/23 15:36:20 kalt Exp $
+**	$Id: support.c,v 1.10 1998/02/17 20:27:53 kalt Exp $
 */
 
 char	*inetntoa(in)
@@ -721,6 +725,26 @@ char *make_version()
 	return mystrdup(ver);
 }
 
+#ifndef HAVE_TRUNCATE
+/* truncate: set a file to a specified length
+ * I don't know of any UNIX that doesn't have truncate, but CYGWIN32 beta18
+ * doesn't have it.  -krys
+ * Replacement version from Dave Miller.
+ */
+int truncate(path, length)
+const char *path;
+size_t length;
+{
+	int fd, res;
+	fd = open(path, O_WRONLY);
+	if (fd == -1)
+		return -1;
+	res = ftruncate(fd, length);
+	close(fd);
+	return res;
+}
+#endif /* HAVE_TRUNCATE */
+
 #if SOLARIS_2_3
 /* 
  * On Solaris 2.3 (SunOS 5.3) systems, gethostbyname() has a bug, it always
