@@ -19,7 +19,7 @@
  */
 
 #ifndef lint
-static  char rcsid[] = "@(#)$Id: ircd.c,v 1.11 1997/09/22 12:18:39 kalt Exp $";
+static  char rcsid[] = "@(#)$Id: ircd.c,v 1.14 1998/02/18 18:41:34 kalt Exp $";
 #endif
 
 #include "os.h"
@@ -229,7 +229,13 @@ time_t	currenttime;
 		cptr = find_name(aconf->name, (aClient *)NULL);
 		if (!cptr)
 			cptr = find_mask(aconf->name, (aClient *)NULL);
-
+		/*
+		** It is not connected, scan clients and see if any matches
+		** a D(eny) line.
+		*/
+		if (find_denied(aconf->name, Class(cltmp)))
+			continue;
+		/* We have a candidate, let's see if it could be the best. */
 		if (!cptr && (Links(cltmp) < MaxLinks(cltmp)) &&
 		    (!con_conf ||
 		     (con_conf->pref > aconf->pref && aconf->pref >= 0) ||
@@ -475,7 +481,8 @@ aClient	*mp;
 	struct	passwd	*p;
 
 	p = getpwuid(getuid());
-	strncpyzt(mp->username, p->pw_name, sizeof(mp->username));
+	strncpyzt(mp->username, (p) ? p->pw_name : "unknown",
+		  sizeof(mp->username));
 	(void)get_my_name(mp, mp->sockhost, sizeof(mp->sockhost)-1);
 	if (mp->name[0] == '\0')
 		strncpyzt(mp->name, mp->sockhost, sizeof(mp->name));
@@ -498,7 +505,8 @@ aClient	*mp;
 	mp->user->flags |= FLAGS_OPER;
 	mp->serv->up = mp->name;
 	mp->user->server = find_server_string(mp->serv->snum);
-	strncpyzt(mp->user->username, p->pw_name, sizeof(mp->user->username));
+	strncpyzt(mp->user->username, (p) ? p->pw_name : "unknown",
+		  sizeof(mp->user->username));
 	(void) strcpy(mp->user->host, mp->name);
 
 	(void)add_to_client_hash_table(mp->name, mp);
