@@ -22,7 +22,7 @@
  */
 
 #ifndef lint
-static  char rcsid[] = "@(#)$Id: s_service.c,v 1.17 1997/10/08 20:20:03 kalt Exp $";
+static  char rcsid[] = "@(#)$Id: s_service.c,v 1.19 1997/12/18 13:33:30 kalt Exp $";
 #endif
 
 #include "os.h"
@@ -568,6 +568,7 @@ char	*parv[];
 	** distribution code is respected.
 	** service type also respected.
 	*/
+	cptr->flags |= FLAGS_CBURST;
 	if (burst & SERVICE_WANT_SERVER)
 	    {
 		int	split;
@@ -652,6 +653,7 @@ char	*parv[];
 			    }
 		    }
 	    }
+	cptr->flags ^= FLAGS_CBURST;
 	return 0;
 }
 #endif
